package BulkFileUploader::CMS;

use Symbol;
use MT::Util qw( epoch2ts encode_url format_ts relative_date );

use Data::Dumper;

sub change_script {
    my ($cb, $app, $tmpl) = @_;

    my $q = $app->param;
    return unless $q->param('__mode') eq 'start_multi_file_upload';

    my $old = <<HERE;
  <script type="text/javascript" src="<$mt:var name="static_uri"$>jquery/jquery.min.js?v=<mt:var name="mt_version_id" escape="URL">"></script>
HERE
    $old = quotemeta($old);

    my $new = <<HERE;
  <script type="text/javascript" src="<mt:var name="static_uri">plugins/BulkFileUploader/jquery/jquery.min.js?v=<mt:var name="mt_version_id" escape="URL">"></script>
HERE
    $$tmpl =~ s/$old/$new/;

    $old = <<HERE;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>jquery/jquery.min.js?v=<mt:var name="mt_version_id" escape="URL">"></script>
HERE
    $old = quotemeta($old);

    $new = <<HERE;
  <script type="text/javascript" src="<mt:var name="static_uri">plugins/BulkFileUploader/jquery/jquery.min.js?v=<mt:var name="mt_version_id" escape="URL">"></script>
HERE
    $$tmpl =~ s/$old/$new/;

}

sub start_multi_file_upload {
    my $app = shift;
    my %param;
    my $q = $app->param;

    _set_start_upload_params( $app, \%param );

    my $plugin = MT->component('BulkFileUploader');
    $param{'skip'} = $plugin->translate('skip');
    $param{'overwrite'} = $plugin->translate('overwrite');
    $param{'uploading'} = $plugin->translate('Uploading ...');

    my $entry_maps = _populate_entry_loop( $app->blog );
    $param{entries} = $entry_maps;

    $param{ rest_day } = $plugin->translate('can use [_1]', rest_day());

    $param{'usage'} = $plugin->translate('usage');
    my $tmpl = 'multi_upload_file.tmpl';
    return $app->build_page($tmpl, \%param);
}

sub _populate_entry_loop {
    my $blog = shift;

    my $plugin = MT->component("BulkFileUploader");
    my $limit = $plugin->get_config_value('limit', 'blog:'.$blog->id);
    $limit = 100 if !$limit;

    my @entries = MT::Entry->load( { blog_id => $blog->id },
                                   { direction => 'descend',
                                     limit => $limit,
                                     sort => 'created_on', } );
    @entries = sort { $b->id <=> $a->id } @entries;
    my @maps;
    my %types;
    foreach my $entry (@entries) {
        my $map = {};
        $map->{entry_id} = $entry->id;
        $map->{entry_title} = $entry->title;
        push @maps, $map;
    }
#    @maps = sort { $a->cat_id <=> $b->cat_id } @maps;
    return \@maps;
}

sub multi_file_upload {
    my $app = shift;

    return $app->error( $app->translate("expiration of a term.") )
        if is_limit();

    if ( my $perms = $app->permissions ) {
        return $app->error( $app->translate("Permission denied.") )
            unless $perms->can_upload;
    }
    else {
        return $app->error( $app->translate("Permission denied.") );
    }

###### ????? ######
#    $app->validate_magic() or return;

    my ( $assets, $bytes_list ) = _upload_file(
        $app,
        require_type => ( $app->param('require_type') || '' ),
        @_,
    );
    return if !defined $assets;
    return $assets if !defined $bytes_list;    # whatever it is

#    complete_insert(
#        $app,
#        asset => $asset,
#        bytes => $bytes,
#    );
    $app->forward("list_asset");
}

sub _upload_file {
    my $app = shift;
    my (%upload_param) = @_;

    my $q = $app->param;
    my ( $fh, $info ) = upload_info($app, 'file');

    my $mimetype;
    if ($info) {
        $mimetype = $info->{'Content-Type'};
    }
    my $has_overwrite = $q->param('has_overwrite');
    my %param = (
        entry_insert => $q->param('entry_insert'),
        middle_path  => $q->param('middle_path'),
        edit_field   => $q->param('edit_field'),
        site_path    => $q->param('site_path'),
        extra_path   => $q->param('extra_path'),
        upload_mode  => $app->mode,
    );
    return start_upload( $app, %param,
        error => $app->translate("Please select a file to upload.") )
        if !$fh && !$has_overwrite;
 my $basename = $q->param('file') || $q->param('fname');
#my $basename = $fh;
    $basename =~ s!\\!/!g;    ## Change backslashes to forward slashes
    $basename =~ s!^.*/!!;    ## Get rid of full directory paths
    if ( $basename =~ m!\.\.|\0|\|! ) {
        return start_upload( $app, %param,
            error => $app->translate( "Invalid filename '[_1]'", $basename )
        );
    }
    if ( my $asset_type = $upload_param{require_type} ) {
        require MT::Asset;
        my $asset_pkg = MT::Asset->handler_for_file($basename);

        my %settings_for = (
            audio => {
                class => 'MT::Asset::Audio',
                error =>
                    $app->translate("Please select an audio file to upload."),
            },
            image => {
                class => 'MT::Asset::Image',
                error => $app->translate("Please select an image to upload."),
            },
            video => {
                class => 'MT::Asset::Video',
                error => $app->translate("Please select a video to upload."),
            },
        );

        if ( my $settings = $settings_for{$asset_type} ) {
            return start_upload( $app, %param, error => $settings->{error} )
                if !$asset_pkg->isa( $settings->{class} );
        }
    }

    my ($blog_id,        $blog,         $fmgr,
        $local_file,     $asset_file,   $base_url,
        $asset_base_url, $relative_url, $relative_path
    );
    if ( $blog_id = $q->param('blog_id') ) {

        $param{blog_id} = $blog_id;
        require MT::Blog;
        $blog = MT::Blog->load($blog_id)
            or return $app->error(
            $app->translate( 'Can\'t load blog #[_1].', $blog_id ) );
        $fmgr = $blog->file_mgr;

        ## Set up the full path to the local file; this path could start
        ## at either the Local Site Path or Local Archive Path, and could
        ## include an extra directory or two in the middle.
        my ( $root_path, $middle_path );
        if ( $q->param('site_path') ) {
            $root_path = $blog->site_path;
        }
        else {
            $root_path = $blog->archive_path;
        }
        return $app->error(
            $app->translate(
                "Before you can upload a file, you need to publish your blog."
            )
        ) unless -d $root_path;

        $relative_path = $q->param('extra_path');
        $middle_path = $q->param('middle_path') || '';
        my $relative_path_save = $relative_path;
        if ( $middle_path ne '' ) {
            $relative_path = $middle_path
                . ( $relative_path ? '/' . $relative_path : '' );
        }
        my $path = $root_path;
        if ($relative_path) {
            if ( $relative_path =~ m!\.\.|\0|\|! ) {
                return start_upload(
                    $app, %param,
                    error => $app->translate(
                        "Invalid extra path '[_1]'",
                        $relative_path
                    )
                );
            }
            $path = File::Spec->catdir( $path, $relative_path );
            ## Untaint. We already checked for security holes in $relative_path.
            ($path) = $path =~ /(.+)/s;
            ## Build out the directory structure if it doesn't exist. DirUmask
            ## determines the permissions of the new directories.
            unless ( $fmgr->exists($path) ) {
                $fmgr->mkpath($path)
                    or return start_upload(
                    $app, %param,
                    error => $app->translate(
                        "Can't make path '[_1]': [_2]", $path,
                        $fmgr->errstr
                    )
                    );
            }
        }

        $relative_url
            = File::Spec->catfile( $relative_path, encode_url($basename) );
        $relative_path
            = $relative_path
            ? File::Spec->catfile( $relative_path, $basename )
            : $basename;
        $asset_file = $q->param('site_path') ? '%r' : '%a';
        $asset_file = File::Spec->catfile( $asset_file, $relative_path );
        $local_file = File::Spec->catfile( $path,       $basename );
        $base_url
            = $app->param('site_path')
            ? $blog->site_url
            : $blog->archive_url;
        $asset_base_url = $app->param('site_path') ? '%r' : '%a';

        ## Untaint. We have already tested $basename and $relative_path for security
        ## issues above, and we have to assume that we can trust the user's
        ## Local Archive Path setting. So we should be safe.
        ($local_file) = $local_file =~ /(.+)/s;

        ## If $local_file already exists, we try to write the upload to a
        ## tempfile, then ask for confirmation of the upload.
        if ( $fmgr->exists($local_file) ) {

            if ($has_overwrite) {

# for overwrite(bring else root)
#------------------------------
                eval { require File::Temp };
                if ($@) {
                    return $app->error(
                        $app->translate(
                            "File with name '[_1]' already exists. (Install "
                                . "the File::Temp Perl module if you would like "
                                . "to be able to overwrite existing uploaded files.)",
                            $basename
                        )
                    );
                }
                my $tmp_dir = $app->config('TempDir');
                my ( $tmp_fh, $tmp_file );
                eval {
                    ( $tmp_fh, $tmp_file )
                        = File::Temp::tempfile( DIR => $tmp_dir );
                };
                if ($@) {    #!$tmp_fh
                    return $app->errtrans(
                        "Error creating temporary file; please check your TempDir "
                            . "setting in your coniguration file (currently '[_1]') "
                            . "this location should be writable.",
                        (     $tmp_dir
                            ? $tmp_dir
                            : '[' . $app->translate('unassigned') . ']'
                        )
                    );
                }
                defined( _write_upload( $fh, $tmp_fh ) )
                    or return $app->error(
                    $app->translate(
                        "File with name '[_1]' already exists; Tried to write "
                            . "to tempfile, but open failed: [_2]",
                        $basename,
                        "$!"
                    )
                    );
                close $tmp_fh;
                my ( $vol, $path, $tmp ) = File::Spec->splitpath($tmp_file);
#------------------------------

#                my $tmp = $q->param('temp');
                if ( $tmp =~ m!([^/]+)$! ) {
                    $tmp = $1;
                }
                else {
                    return $app->error(
                        $app->translate(
                            "Invalid temp file name '[_1]'", $tmp
                        )
                    );
                }

                my $tmp_dir = $app->config('TempDir');
                my $tmp_file = File::Spec->catfile( $tmp_dir, $tmp );
#                if ( $q->param('overwrite_yes') ) {
                    $fh = gensym();
                    open $fh, $tmp_file
                        or return $app->error(
                        $app->translate(
                            "Error opening '[_1]': [_2]",
                            $tmp_file, "$!"
                        )
                        );

#                }
#                else {
#                    if ( -e $tmp_file ) {
#                        unlink($tmp_file)
#                            or return $app->error(
#                            $app->translate(
#                                "Error deleting '[_1]': [_2]", $tmp_file,
#                                "$!"
#                            )
#                            );
#                    }
#                    return start_upload($app);
#                }
            } else {
                return;
            }
#            else {
#                eval { require File::Temp };
#                if ($@) {
#                    return $app->error(
#                        $app->translate(
#                            "File with name '[_1]' already exists. (Install "
#                                . "File::Temp if you'd like to be able to overwrite "
#                                . "existing uploaded files.)",
#                            $basename
#                        )
#                    );
#                }
#                my $tmp_dir = $app->config('TempDir');
#                my ( $tmp_fh, $tmp_file );
#                eval {
#                    ( $tmp_fh, $tmp_file )
#                        = File::Temp::tempfile( DIR => $tmp_dir );
#                };
#                if ($@) {    #!$tmp_fh
#                    return $app->errtrans(
#                        "Error creating temporary file; please check your TempDir "
#                            . "setting in your coniguration file (currently '[_1]') "
#                            . "this location should be writable.",
#                        (     $tmp_dir
#                            ? $tmp_dir
#                            : '[' . $app->translate('unassigned') . ']'
#                        )
#                    );
#                }
#                defined( _write_upload( $fh, $tmp_fh ) )
#                    or return $app->error(
#                    $app->translate(
#                        "File with name '[_1]' already exists; Tried to write "
#                            . "to tempfile, but open failed: [_2]",
#                        $basename,
#                        "$!"
#                    )
#                    );
#                close $tmp_fh;
#                my ( $vol, $path, $tmp ) = File::Spec->splitpath($tmp_file);
#                return $app->load_tmpl(
#                    'dialog/asset_replace.tmpl',
#                    {   temp         => $tmp,
#                        extra_path   => $relative_path_save,
#                        site_path    => scalar $q->param('site_path'),
#                        asset_select => $q->param('asset_select'),
#                        entry_insert => $q->param('entry_insert'),
#                        edit_field   => $app->param('edit_field'),
#                        middle_path  => $middle_path,
#                        fname        => $basename
#                    }
#                );
#            }
        }
    }
#    else {
#        $blog_id        = 0;
#        $asset_base_url = '%s/support/uploads';
#        $base_url       = $app->static_path . 'support/uploads';
#        $param{support_path}
#            = File::Spec->catdir( $app->static_file_path, 'support',
#            'uploads' );
#
#        require MT::FileMgr;
#        $fmgr = MT::FileMgr->new('Local');
#        unless ( $fmgr->exists( $param{support_path} ) ) {
#            $fmgr->mkpath( $param{support_path} );
#            unless ( $fmgr->exists( $param{support_path} ) ) {
#                return $app->error(
#                    $app->translate(
#                        "Could not create upload path '[_1]': [_2]",
#                        $param{support_path},
#                        $fmgr->errstr
#                    )
#                );
#            }
#        }
#
#        require File::Basename;
#        my ( $stem, undef, $type )
#            = File::Basename::fileparse( $basename, qr/\.[A-Za-z0-9]+$/ );
#        my $unique_stem = $stem;
#        $local_file = File::Spec->catfile( $param{support_path},
#            $unique_stem . $type );
#        my $i = 1;
#        while ( $fmgr->exists($local_file) ) {
#            $unique_stem = join q{-}, $stem, $i++;
#            $local_file = File::Spec->catfile( $param{support_path},
#                $unique_stem . $type );
#        }
#
#        my $unique_basename = $unique_stem . $type;
#        $relative_path = $unique_basename;
#        $relative_url  = encode_url($unique_basename);
#        $asset_file    = File::Spec->catfile( '%s', 'support', 'uploads',
#            $unique_basename );
#    }

    if ( my $deny_exts = $app->config->DeniedAssetFileExtensions ) {
        my @deny_exts = map { if ( $_ =~ m/^\./ ) { qr/$_/i } else { qr/\.$_/i } } split '\s?,\s?', $deny_exts;
        my @ret = File::Basename::fileparse( $basename, @deny_exts );
        if ( $ret[2] ) {
            return $app->error($app->translate('The file([_1]) you uploaded is not allowed.', $basename));
        }

    }

    if ( my $allow_exts = $app->config('AssetFileExtensions') ) {
        my @allow_exts = map { if ( $_ =~ m/^\./ ) { qr/$_/i } else { qr/\.$_/i } } split '\s?,\s?', $allow_exts;
        my @ret = File::Basename::fileparse( $basename, @allow_exts );
        unless ( $ret[2] ) {
            return $app->error($app->translate('The file([_1]) you uploaded is not allowed.', $basename));
        }
    }

    require MT::Image;
    my ( $w, $h, $id, $write_file ) = MT::Image->check_upload(
        Fh     => $fh,
        Fmgr   => $fmgr,
        Local  => $local_file,
        Max    => $upload_param{max_size},
        MaxDim => $upload_param{max_image_dimension}
    );

    return $app->error( MT::Image->errstr )
        unless $write_file;

    ## File does not exist, or else we have confirmed that we can overwrite.
    my $umask = oct $app->config('UploadUmask');
    my $old   = umask($umask);
    defined( my $bytes = $write_file->() )
        or return $app->error(
        $app->translate(
            "Error writing upload to '[_1]': [_2]", $local_file,
            $fmgr->errstr
        )
        );
    umask($old);

    ## Close up the filehandle.
    close $fh;

    ## If we are overwriting the file, that means we still have a temp file
    ## lying around. Delete it.
    if ( $q->param('overwrite_yes') ) {
        my $tmp = $q->param('temp');
        if ( $tmp =~ m!([^/]+)$! ) {
            $tmp = $1;
        }
        else {
            return $app->error(
                $app->translate( "Invalid temp file name '[_1]'", $tmp ) );
        }
        my $tmp_file = File::Spec->catfile( $app->config('TempDir'), $tmp );
        unlink($tmp_file)
            or return $app->error(
            $app->translate( "Error deleting '[_1]': [_2]", $tmp_file, "$!" )
            );
    }

    ## We are going to use $relative_path as the filename and as the url passed
    ## in to the templates. So, we want to replace all of the '\' characters
    ## with '/' characters so that it won't look like backslashed characters.
    ## Also, get rid of a slash at the front, if present.
    $relative_path =~ s!\\!/!g;
    $relative_path =~ s!^/!!;
    $relative_url  =~ s!\\!/!g;
    $relative_url  =~ s!^/!!;
    my $url = $base_url;
    $url .= '/' unless $url =~ m!/$!;
    $url .= $relative_url;
    my $asset_url = $asset_base_url . '/' . $relative_url;

    require File::Basename;
    my $local_basename = File::Basename::basename($local_file);
    my $ext
        = ( File::Basename::fileparse( $local_file, qr/[A-Za-z0-9]+$/ ) )[2];

    require MT::Asset;
    my $asset_pkg = MT::Asset->handler_for_file($local_basename);
    my $is_image  = 0;
    if ( defined($w) && defined($h) ) {
        $is_image = 1
            if $asset_pkg->isa('MT::Asset::Image');
    }
    else {

        # rebless to file type
        $asset_pkg = 'MT::Asset'
            if $asset_pkg->isa('MT::Asset::Image');
    }
    return $app->errtrans('Uploaded file is not an image.')
        if !$is_image
            && exists( $upload_param{require_type} )
            && $upload_param{require_type} eq 'image';
    my $asset;
    if (!(  $asset = $asset_pkg->load(
                { file_path => $asset_file, blog_id => $blog_id },
                { binary => { file_path => 1 } }
            )
        )
        )
    {
        $asset = $asset_pkg->new();
        $asset->file_path($asset_file);
        $asset->file_name($local_basename);
        $asset->file_ext($ext);
        $asset->blog_id($blog_id);
        $asset->created_by( $app->user->id );
    }
    else {
        $asset->modified_by( $app->user->id );
    }
    my $original = $asset->clone;
    $asset->url($asset_url);
    if ($is_image) {
        $asset->image_width($w);
        $asset->image_height($h);
    }
    $asset->mime_type($mimetype) if $mimetype;

    if ( $q->param('tags') ) {
        require MT::Tag;
        my $tag_delim = chr( $app->user->entry_prefs->{tag_delim} );
        my @tags = MT::Tag->split( $tag_delim, $q->param('tags') );
        $asset->set_tags(@tags);
    }

    $asset->save;

    my $entry_id = $q->param('entry_id') ? $q->param('entry_id') : $q->param('entry_id2');
    if ( $entry_id ) {
        require MT::Asset;
        require MT::ObjectAsset;

        my $obj_asset = MT::ObjectAsset->load(
            {   asset_id  => $asset->id,
                object_ds => 'entry',
                object_id => $entry_id
            }
        );
        unless ($obj_asset) {
            my $obj_asset = new MT::ObjectAsset;
            $obj_asset->blog_id( $blog_id );
            $obj_asset->asset_id( $asset->id );
            $obj_asset->object_ds('entry');
            $obj_asset->object_id( $entry_id );
            $obj_asset->save;
        }
    }

    $app->run_callbacks( 'cms_post_save.asset', $app, $asset, $original );

    if ($is_image) {
        $app->run_callbacks(
            'cms_upload_file.' . $asset->class,
            File  => $local_file,
            file  => $local_file,
            Url   => $url,
            url   => $url,
            Size  => $bytes,
            size  => $bytes,
            Asset => $asset,
            asset => $asset,
            Type  => 'image',
            type  => 'image',
            Blog  => $blog,
            blog  => $blog
        );
        $app->run_callbacks(
            'cms_upload_image',
            File       => $local_file,
            file       => $local_file,
            Url        => $url,
            url        => $url,
            Size       => $bytes,
            size       => $bytes,
            Asset      => $asset,
            asset      => $asset,
            Height     => $h,
            height     => $h,
            Width      => $w,
            width      => $w,
            Type       => 'image',
            type       => 'image',
            ImageType  => $id,
            image_type => $id,
            Blog       => $blog,
            blog       => $blog
        );
    }
    else {
        $app->run_callbacks(
            'cms_upload_file.' . $asset->class,
            File  => $local_file,
            file  => $local_file,
            Url   => $url,
            url   => $url,
            Size  => $bytes,
            size  => $bytes,
            Asset => $asset,
            asset => $asset,
            Type  => 'file',
            type  => 'file',
            Blog  => $blog,
            blog  => $blog
        );
    }

#    push @assets, $asset;
#    push @bytes_list, $bytes;
#}
#    return ( \@assets, \@bytes_list );
    return ( $asset, $bytes );
}

#
# no use
#
#sub start_upload {
#    my $app = shift;
#
#    $app->add_breadcrumb( $app->translate('Upload File') );
#    my %param;
#    %param = @_ if @_;
#
#    _set_start_upload_params( $app, \%param );
#
#    for my $field (
#        qw( entry_insert edit_field upload_mode require_type
#        asset_select )
#        )
#    {
#        $param{$field} ||= $app->param($field);
#    }
#
#    $app->load_tmpl( 'dialog/asset_upload.tmpl', \%param );
#}

sub _set_start_upload_params {
    my $app = shift;
    my ($param) = @_;

    if ( my $perms = $app->permissions ) {
        return $app->error( $app->translate("Permission denied.") )
            unless $perms->can_upload;
        my $blog_id = $app->param('blog_id');
        require MT::Blog;
        my $blog = MT::Blog->load($blog_id)
            or return $app->error(
            $app->translate( 'Can\'t load blog #[_1].', $blog_id ) );

        $param->{enable_archive_paths} = $blog->column('archive_path');
        $param->{local_site_path}      = $blog->site_path;
        $param->{local_archive_path}   = $blog->archive_path;
        my $label_path;
        if ( $param->{enable_archive_paths} ) {
            $label_path = $app->translate('Archive Root');
        }
        else {
            $label_path = $app->translate('Site Root');
        }
        my @extra_paths;
        my $date_stamp = epoch2ts( $blog, time );
        $date_stamp =~ s!^(\d\d\d\d)(\d\d)(\d\d).*!$1/$2/$3!;
        my $path_hash = {
            path  => $date_stamp,
            label => '<' . $label_path . '>' . '/' . $date_stamp,
        };

        if ( exists( $param->{middle_path} )
            && ( $date_stamp eq $param->{middle_path} ) )
        {
            $path_hash->{selected} = 1;
            delete $param->{archive_path};
        }
        push @extra_paths, $path_hash;
        $param->{extra_paths} = \@extra_paths;
        $param->{refocus}     = 1;
        $param->{missing_paths}
            = (    ( defined $blog->site_path || defined $blog->archive_path )
                && ( -d $blog->site_path || -d $blog->archive_path ) )
            ? 0
            : 1;

        if ( $param->{missing_paths} ) {
            if ($app->user->is_superuser
                || $app->run_callbacks(
                    'cms_view_permission_filter.blog',
                    $app, $blog_id, $blog
                )
                )
            {
                $param->{have_permissions} = 1;
            }
        }

        $param->{enable_destination} = 1;

        my $data = $app->_build_category_list(
            blog_id => $blog_id,
            markers => 1,
            type    => 'folder',
        );
        my $top_cat  = -1;
        my $cat_tree = [
            {   id       => -1,
                label    => '/',
                basename => '/',
                path     => [],
            }
        ];
        foreach (@$data) {
            next unless exists $_->{category_id};
            $_->{category_path_ids} ||= [];
            unshift @{ $_->{category_path_ids} }, -1;
            push @$cat_tree,
                {
                id       => $_->{category_id},
                label    => $_->{category_label} . '/',
                basename => $_->{category_basename} . '/',
                path     => $_->{category_path_ids} || [],
                };
        }
        $param->{category_tree} = $cat_tree;
    }
    else {
        $param->{local_site_path}    = '';
        $param->{local_archive_path} = '';
    }

    $param;
}

sub _write_upload {
    my ( $upload_fh, $dest_fh ) = @_;
    my $fh = gensym();
    if ( ref($dest_fh) eq 'GLOB' ) {
        $fh = $dest_fh;
    }
    else {
        open $fh, ">$dest_fh" or return;
    }
    binmode $fh;
    binmode $upload_fh;
    my ( $bytes, $data ) = (0);
    while ( my $len = read $upload_fh, $data, 8192 ) {
        print $fh $data;
        $bytes += $len;
    }
    close $fh;
    $bytes;
}

sub upload_info {
    my ($app, $param_name) = @_;
    my $q            = $app->param;

    my ( $fh, $info, $no_upload );
    if ( $ENV{MOD_PERL} ) {
        if ( my $up = $q->upload($param_name) ) {
#$up = decode_base64($up);
            $fh        = $up->fh;
            $info      = $up->info;
            $no_upload = !$up->size;
        }
        else {
            $no_upload = 1;
        }
    }
    else {
        ## Older versions of CGI.pm didn't have an 'upload' method.
        eval { $fh = $q->upload($param_name) };
        if ( $@ && $@ =~ /^Undefined subroutine/ ) {
            $fh = $q->param($param_name);
        }
        $no_upload = !$fh;
        $info      = $q->uploadInfo($fh);
    }

    return if $no_upload;
    return ( $fh, $info );
}

sub init_request {
    my $plugin = MT->component("BulkFileUploader");
    my $file_upload = $plugin->get_config_value('file_upload', 'system') || ',';
    if ($file_upload eq ',') {
        $plugin->set_config_value('file_upload', time(), 'system');
    }
}

sub is_limit {
    return (rest_day() > 0) ? 0 : 1;
}

sub rest_day {
    my $plugin = MT->component("BulkFileUploader");
    my $file_upload = $plugin->get_config_value('file_upload', 'system') || '';
    my $duration = time() - $file_upload;
    my $rest_day = 10 - int($duration / (24 * 3600));
    return ($rest_day > 0) ? $rest_day : 0;
}

1;
